@@ -1,19 +1,22 @@
 package Mail::Milter::Authentication::Protocol::SMTP;
+use 5.20.0;
 use strict;
 use warnings;
-use version; our $VERSION = version->declare('v1.1.0');
-
-use English qw{ -no_match_vars };
+use Mail::Milter::Authentication::Pragmas;
+# ABSTRACT: SMTP protocol handling
+our $VERSION = '3.20240701'; # VERSION
+use Digest::MD5 qw{ md5_hex };
 use Email::Date::Format qw{ email_date };
-use IO::Socket;
 use IO::Socket::INET;
 use IO::Socket::UNIX;
-use Digest::MD5 qw{ md5_hex };
+use IO::Socket;
 use Net::IP;
-use Sys::Syslog qw{:standard :macros};
 
-use Mail::Milter::Authentication::Constants qw{ :all };
-use Mail::Milter::Authentication::Config;
+sub register_metrics {
+    return {
+        'mail_processed_total' => 'Number of emails processed',
+    };
+}
 
 sub get_smtp_config {
     my ( $self ) = @_;
@@ -40,7 +43,6 @@ sub smtp_status {
     my ( $self, $status ) = @_;
     my $smtp = $self->{'smtp'};
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':' . $status . '(' . $self->{'count'} . '.' . $smtp->{'count'} . ')';
-    return;
 }
 
 sub smtp_init {
@@ -58,11 +60,9 @@ sub smtp_init {
     $handler->set_symbol( 'C', 'i', $self->smtp_queue_id() );
 
     $smtp->{'count'}++ ;
-    $handler->dbgout( 'SMTP Transaction count', $self->{'count'} . '.' . $smtp->{'count'} , LOG_INFO );
+    $handler->dbgout( 'SMTP Transaction count', $self->{'count'} . '.' . $smtp->{'count'} , LOG_DEBUG );
 
     $smtp->{'init_required'} = 0;
-
-    return;
 }
 
 sub protocol_process_request {
@@ -71,6 +71,10 @@ sub protocol_process_request {
     my $handler = $self->{'handler'}->{'_Handler'};
     $handler->top_setup_callback();
 
+    my $config = $self->{ 'config' };
+    my $seconds = $config->{'content_timeout'} // 300;
+    $handler->set_overall_timeout( $seconds * 1000000 );
+
     $self->{'smtp'} = {
         'fwd_helo_host'    => undef,
         'fwd_connect_ip'   => undef,
@@ -141,6 +145,15 @@ sub protocol_process_request {
 
         $self->logdebug( "receive command $command" );
 
+        if ( exists ( $smtp_config->{ 'debug_triggers' } ) ) {
+            my $triggers = $smtp_config->{ 'debug_triggers' };
+            foreach my $trigger ( @$triggers ) {
+                if ( $command =~ /$trigger/ ) {
+                    $self->enable_extra_debugging();
+                }
+            }
+        }
+
         my $returncode = SMFIS_CONTINUE;
 
         if ( $uccommand =~ /^EHLO/ ) {
@@ -166,6 +179,7 @@ sub protocol_process_request {
             $self->smtp_command_rset( $command );
         }
         elsif ( $uccommand =~ /^DATA/ ) {
+            $handler->set_overall_timeout( $seconds * 1000000 );
             $self->smtp_command_data( $command );
         }
         elsif ( $uccommand =~ /^QUIT/ ){
@@ -180,13 +194,13 @@ sub protocol_process_request {
         }
 
     }
+    $handler->clear_overall_timeout();
 
     $self->smtp_status('smtp.close');
 
     $self->close_destination_socket();
 
     delete $self->{'smtp'};
-    return;
 }
 
 sub smtp_queue_id {
@@ -208,6 +222,44 @@ sub command_param {
     return $p;
 }
 
+# Splits the given command parameter line into a list of separate
+# path (<foo@example.com>) and esmtp-param (KEYWORD=VALUE) strings.
+sub split_command_params {
+    my ( $line ) = @_;
+    $line =~ s/^\s+//;
+    my $path_end = find_path_end( $line ) || length( $line );
+    my $path = substr( $line, 0, $path_end );
+    $line = substr( $line, $path_end );
+    $line =~ s/^\s+//;
+    return ( $path, split( /\s+/, $line ) );
+}
+
+sub find_path_end {
+    my ( $input ) = @_;
+    my $len = length( $input );
+    my @qstack;
+    for ( my $i = 0; $i < $len; $i++ ) {
+        my $c = substr( $input, $i, 1 );
+        if ( @qstack && $c eq $qstack[-1] ) {
+            # end of a quoted string
+            pop @qstack;
+        } elsif ( $c eq '"' ) {
+            # start of a "" quoted string
+            push @qstack, '"';
+        } elsif ( $c eq '<' && ( !@qstack || $qstack[-1] eq '>' ) ) {
+            # start of a <> quoted string (possibly inside of another <>)
+            push @qstack, '>';
+        } elsif ( $c eq '\\' && @qstack ) {
+            # escaped character within a quoted string
+            $i++;
+        } elsif ( $c eq ' ' && !@qstack ) {
+            # found the end
+            return $i;
+        }
+    }
+    return ( @qstack ? undef : $len );
+}
+
 sub smtp_command_lhlo {
     my ( $self, $command ) = @_;
     my $smtp = $self->{'smtp'};
@@ -233,7 +285,6 @@ sub smtp_command_lhlo {
     print $socket "250-PIPELINING\r\n";
     print $socket "250-ENHANCEDSTATUSCODES\r\n";
     print $socket "250 8BITMIME\r\n";
-    return;
 }
 
 sub smtp_command_ehlo {
@@ -259,7 +310,6 @@ sub smtp_command_ehlo {
     print $socket "250-PIPELINING\r\n";
     print $socket "250-ENHANCEDSTATUSCODES\r\n";
     print $socket "250 8BITMIME\r\n";
-    return;
 }
 
 sub smtp_command_helo {
@@ -276,7 +326,6 @@ sub smtp_command_helo {
     }
     $smtp->{'helo_host'} = command_param( $command,5 );
     print $socket "250 " . $smtp->{'server_name'} . " Hi " . $smtp->{'helo_host'} . "\r\n";
-    return;
 }
 
 sub smtp_command_xforward {
@@ -322,7 +371,6 @@ sub smtp_command_xforward {
         }
     }
     print $socket "250 2.0.0 Ok\r\n";
-    return;
 }
 
 sub smtp_command_rset {
@@ -348,8 +396,15 @@ sub smtp_command_rset {
     $self->smtp_init();
 
     my $smtp_conf = $self->get_smtp_config();
+
     my $handler = $self->{'handler'}->{'_Handler'};
-        if ( $smtp_conf->{'pipeline_limit'} ) {
+    if ( $handler->{'exit_on_close'} ) {
+            $smtp->{'last_command'} = 1;
+            $handler->dbgout( 'EXIT ON CLOSE REQUESTED', 'closing on RSET', LOG_INFO );
+            print $socket "421 4.3.2 Pipeline limit reached\r\n";
+            return;
+    }
+    if ( $smtp_conf->{'pipeline_limit'} ) {
         my $count = $smtp->{'count'};
         my $limit = $smtp_conf->{'pipeline_limit'};
         if ( $count > $limit ) {
@@ -360,8 +415,6 @@ sub smtp_command_rset {
         }
     }
     print $socket "250 2.0.0 Ok\r\n";
-
-    return;
 }
 
 sub smtp_command_mailfrom {
@@ -392,15 +445,18 @@ sub smtp_command_mailfrom {
         $ip = substr( $ip, 5 );
     }
 
-    $self->logdebug( "Inbound IP Address $ip" );
-    $returncode = $handler->top_connect_callback( $host, Net::IP->new( $ip ) );
+    # Do connection remapping first
+    $handler->remap_connect_callback( $host, Net::IP->new( $ip ) );
+    $handler->remap_helo_callback( $helo );
+
+    $self->logdebug( "Inbound IP Address " . $handler->{'ip_object'}->ip() );
+    $returncode = $handler->top_connect_callback( $host, $handler->{'ip_object'} );
     if ( $returncode == SMFIS_CONTINUE ) {
-        $returncode = $handler->top_helo_callback( $helo );
+        $returncode = $handler->top_helo_callback( $handler->{'helo_name'} );
         if ( $returncode == SMFIS_CONTINUE ) {
             my $envfrom = command_param( $command,10 );
             $smtp->{'mail_from'} = $envfrom;
-            $envfrom =~ s/ BODY=8BITMIME$//;
-            $returncode = $handler->top_envfrom_callback( $envfrom );
+            $returncode = $handler->top_envfrom_callback( split_command_params( $envfrom ) );
             if ( $returncode == SMFIS_CONTINUE ) {
                 $smtp->{'has_mail_from'} = 1;
                 print $socket "250 2.0.0 Ok\r\n";
@@ -410,6 +466,11 @@ sub smtp_command_mailfrom {
                 $self->loginfo ( "SMTPReject: $reject_reason" );
                 print $socket $reject_reason . "\r\n";
             }
+            elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+                $handler->clear_defer_mail();
+                $self->loginfo ( "SMTPDefer: $defer_reason" );
+                print $socket $defer_reason . "\r\n";
+            }
             else {
                 print $socket "451 4.0.0 MAIL - That's not right\r\n";
             }
@@ -419,8 +480,13 @@ sub smtp_command_mailfrom {
             $self->loginfo ( "SMTPReject: $reject_reason" );
             print $socket $reject_reason . "\r\n";
         }
+        elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+            $handler->clear_defer_mail();
+            $self->loginfo ( "SMTPDefer: $defer_reason" );
+            print $socket $defer_reason . "\r\n";
+        }
         else {
-            print $socket "451 4.0.0 HELO - That's not right\r\n";
+            print $socket "451 4.0.1 HELO - That's not right\r\n";
         }
     }
     elsif ( my $reject_reason = $handler->get_reject_mail() ) {
@@ -428,11 +494,14 @@ sub smtp_command_mailfrom {
         $self->loginfo ( "SMTPReject: $reject_reason" );
         print $socket $reject_reason . "\r\n";
     }
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->clear_defer_mail();
+        $self->loginfo ( "SMTPDefer: $defer_reason" );
+        print $socket $defer_reason . "\r\n";
+    }
     else {
-        print $socket "451 4.0.0 Connection - That's not right\r\n";
+        print $socket "451 4.0.2 Connection - That's not right\r\n";
     }
-
-    return;
 }
 
 sub smtp_command_rcptto {
@@ -449,7 +518,7 @@ sub smtp_command_rcptto {
     }
     my $envrcpt = command_param( $command,8 );
     push @{ $smtp->{'rcpt_to'} }, $envrcpt;
-    my $returncode = $handler->top_envrcpt_callback( $envrcpt );
+    my $returncode = $handler->top_envrcpt_callback( split_command_params( $envrcpt ) );
     if ( $returncode == SMFIS_CONTINUE ) {
         push @{ $smtp->{'lmtp_rcpt'} }, $envrcpt;
         print $socket "250 2.0.0 Ok\r\n";
@@ -459,11 +528,14 @@ sub smtp_command_rcptto {
         $self->loginfo ( "SMTPReject: $reject_reason" );
         print $socket $reject_reason . "\r\n";
     }
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->clear_defer_mail();
+        $self->loginfo ( "SMTPDefer: $defer_reason" );
+        print $socket $defer_reason . "\r\n";
+    }
     else {
-        print $socket "451 4.0.0 That's not right\r\n";
+        print $socket "451 4.0.3 That's not right\r\n";
     }
-
-    return;
 }
 
 sub smtp_command_data {
@@ -491,6 +563,7 @@ sub smtp_command_data {
         alarm( $smtp->{'smtp_timeout_in'} );
         HEADERS:
         while ( my $dataline = <$socket> ) {
+            $self->extra_debugging( "RAW DEBUG: ". $dataline );
             alarm( 0 );
             $dataline =~ s/\r?\n$//;
             if ( $dataline eq '.' ) {
@@ -505,6 +578,7 @@ sub smtp_command_data {
                 last HEADERS;
             }
             push @header_split, $dataline;
+            alarm( $smtp->{'smtp_timeout_in'} );
         }
     };
     if ( my $error = $@ ) {
@@ -524,10 +598,14 @@ sub smtp_command_data {
             if ( $value ) {
                 push @{ $smtp->{'headers'} } , $value;
                 my ( $hkey, $hvalue ) = split ( ':', $value, 2 );
-                $hvalue =~ s/^ //;
-                my $returncode = $handler->top_header_callback( $hkey, $hvalue );
-                if ( $returncode != SMFIS_CONTINUE ) {
-                    $fail = 1;
+                $hvalue =~ s/^\s+//;
+                $hkey =~ s/^\s+//;
+                $hkey =~ s/\s+$//;
+                if ( ! $fail ) {
+                    my $returncode = $handler->top_header_callback( $hkey, $hvalue, $value );
+                    if ( $returncode != SMFIS_CONTINUE ) {
+                        $fail = 1;
+                    }
                 }
             }
             $value = $header_line;
@@ -536,34 +614,78 @@ sub smtp_command_data {
     if ( $value ) {
         push @{ $smtp->{'headers'} } , $value;
         my ( $hkey, $hvalue ) = split ( ':', $value, 2 );
-        $hvalue =~ s/^ //;
-        my $returncode = $handler->top_header_callback( $hkey, $hvalue );
+        $hvalue =~ s/^\s+//;
+        $hkey =~ s/^\s+//;
+        $hkey =~ s/\s+$//;
+        if ( ! $fail ) {
+            my $returncode = $handler->top_header_callback( $hkey, $hvalue, $value );
+            if ( $returncode != SMFIS_CONTINUE ) {
+                $fail = 1;
+            }
+        }
+    }
+    if ( ! $fail ) {
+        $returncode = $handler->top_eoh_callback();
         if ( $returncode != SMFIS_CONTINUE ) {
             $fail = 1;
         }
     }
-    $returncode = $handler->top_eoh_callback();
-    if ( $returncode != SMFIS_CONTINUE ) {
-        $fail = 1;
+
+    my $smtp_conf = $self->get_smtp_config();
+
+    my $chunk_limit = 1048576; # Process in chunks no larger than...
+    if ( exists ( $smtp_conf->{ 'chunk_limit' } ) ) {
+        $chunk_limit = $smtp_conf->{ 'chunk_limit' };
+    }
+
+    my $temp_file;
+    if ( exists ( $smtp_conf->{ 'temp_dir' } ) ) {
+        # TODO use spool_dir to create this
+        # add config option to en/disable this
+        # allow dir to be overridden
+        $temp_file = File::Temp->new( DIR => $smtp_conf->{ 'temp_dir' } );
     }
 
     $self->smtp_status('smtp.i.body');
+    my $body_chunk = q{};
     if ( ! $done ) {
         eval {
             alarm( $smtp->{'smtp_timeout_in'} );
             DATA:
             while ( my $dataline = <$socket> ) {
+                $self->extra_debugging( "RAW DEBUG: ". $dataline );
                 alarm( 0 );
                 last DATA if $dataline =~  /^\.\r\n/;
                 # Handle transparency
                 if ( $dataline =~ /^\./ ) {
                     $dataline = substr( $dataline, 1 );
                 }
-                $body .= $dataline;
+
+                if ( $temp_file ) {
+                    print $temp_file $dataline;
+                }
+                else {
+                   $body .= $dataline;
+                }
+
+                if ( length( $body_chunk ) + length( $dataline ) > $chunk_limit ) {
+                    $returncode = $handler->top_body_callback( $body_chunk );
+                    if ( $returncode != SMFIS_CONTINUE ) {
+                        $fail = 1;
+                    }
+                    $body_chunk = q{};
+                }
+
+                $body_chunk .= $dataline;
+
+                alarm( $smtp->{'smtp_timeout_in'} );
             }
-            $returncode = $handler->top_body_callback( $body );
-            if ( $returncode != SMFIS_CONTINUE ) {
-                $fail = 1;
+            if ( ! $fail ) {
+                $returncode = $handler->top_body_callback( $body_chunk );
+                if ( $returncode != SMFIS_CONTINUE ) {
+                    $fail = 1;
+                }
+                $body_chunk = q{};
             }
         };
         if ( my $error = $@ ) {
@@ -574,17 +696,33 @@ sub smtp_command_data {
         alarm( 0 );
     }
 
-    $returncode = $handler->top_eom_callback();
-    if ( $returncode != SMFIS_CONTINUE ) {
-        $fail = 1;
+    if ( ! $fail ) {
+        $returncode = $handler->top_eom_callback();
+        if ( $returncode != SMFIS_CONTINUE ) {
+            $fail = 1;
+        }
     }
+
     $self->smtp_status('smtp.i.data.received');
 
     if ( ! $fail ) {
-        $smtp->{'body'} = $body;
+
+        if ( $temp_file ) {
+            $smtp->{'spool'} = $temp_file;
+        }
+        else {
+            $smtp->{'body'} = $body;
+        }
 
         if ( $self->smtp_forward_to_destination() ) {
 
+            if ( my $reject_reason = $handler->get_quarantine_mail() ) {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'quarantined' } );
+            }
+            else {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
+            }
+
             $handler->dbgout( 'Accept string', $smtp->{'string'}, LOG_INFO );
             $smtp->{'has_data'} = 1;
 
@@ -599,20 +737,19 @@ sub smtp_command_data {
         }
         else {
             $self->logerror( "SMTP Mail Rejected" );
-            my $error =  '451 4.0.0 That\'s not right';
+            my $error =  '451 4.0.4 That\'s not right';
             my $upstream_error = $smtp->{'string'};
-            if ( $upstream_error =~ /^451 / ) {
+            if ( $upstream_error =~ /^4\d\d / ) {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred' } );
                 $error = $upstream_error;
             }
-            elsif ( $upstream_error =~ /^554 / ) {
-                # Also pass back rejects
-                $error = $upstream_error;
-            }
-            elsif ( $upstream_error =~ /^501 / ) {
+            elsif ( $upstream_error =~ /^5\d\d / ) {
                 # Also pass back rejects
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'rejected' } );
                 $error = $upstream_error;
             }
             else {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred_error' } );
                 $error .= ': ' . $upstream_error;
             }
             if ( $smtp->{'using_lmtp'} ) {
@@ -626,6 +763,7 @@ sub smtp_command_data {
         }
     }
     elsif ( my $reject_reason = $handler->get_reject_mail() ) {
+        $handler->metric_count( 'mail_processed_total', { 'result' => 'rejected' } );
         $handler->clear_reject_mail();
         if ( $smtp->{'using_lmtp'} ) {
             foreach my $rcpt_to ( @{ $smtp->{'lmtp_rcpt'} } ) {
@@ -638,14 +776,29 @@ sub smtp_command_data {
             print $socket $reject_reason . "\r\n";
         }
     }
-    else {
+    elsif ( my $defer_reason = $handler->get_defer_mail() ) {
+        $handler->metric_count( 'mail_processed_total', { 'result' => 'defered' } );
+        $handler->clear_defer_mail();
         if ( $smtp->{'using_lmtp'} ) {
             foreach my $rcpt_to ( @{ $smtp->{'lmtp_rcpt'} } ) {
-                print $socket "451 4.0.0 That's not right\r\n";
+                $self->loginfo ( "SMTPDefer: $defer_reason" );
+                print $socket $defer_reason . "\r\n";
             }
         }
         else {
-            print $socket "451 4.0.0 That's not right\r\n";
+            $self->loginfo ( "SMTPDefer: $defer_reason" );
+            print $socket $defer_reason . "\r\n";
+        }
+    }
+    else {
+        $handler->metric_count( 'mail_processed_total', { 'result' => 'deferred_error' } );
+        if ( $smtp->{'using_lmtp'} ) {
+            foreach my $rcpt_to ( @{ $smtp->{'lmtp_rcpt'} } ) {
+                print $socket "451 4.0.5 That's not right\r\n";
+            }
+        }
+        else {
+            print $socket "451 4.0.6 That's not right\r\n";
         }
     }
     $self->smtp_status('smtp.i.data.done');
@@ -663,7 +816,11 @@ sub smtp_command_data {
     $smtp->{'string'}           = q{};
     $self->{'handler'}->{'_Handler'}->top_close_callback();
     $smtp->{'init_required'}    = 1;
-    return;
+
+    if ( $handler->{'exit_on_close'} ) {
+            $smtp->{'last_command'} = 1;
+            $handler->dbgout( 'EXIT ON CLOSE REQUESTED', 'NO MORE SMTP', LOG_INFO );
+    }
 }
 
 sub smtp_insert_received_header {
@@ -699,7 +856,6 @@ sub smtp_insert_received_header {
     );
 
     splice @{ $smtp->{'headers'} }, 0, 0, 'Received: '. $value;
-    return;
 }
 
 sub smtp_forward_to_destination {
@@ -804,14 +960,32 @@ sub smtp_forward_to_destination {
     }
     $email .= "\r\n";
 
-    my $body = $smtp->{'body'};
-    $body =~ s/\015?\012/\015\012/g;
-    $email .= $body;
+    my $spool = $smtp->{'spool'};
+    if ( $spool ) {
 
-    # Handle transparency
-    $email =~ s/\015\012\./\015\012\.\./g;
+        # Handle transparency - should not be any in headers, but for completeness
+        $email =~ s/\015\012\./\015\012\.\./g;
 
-    print $sock $email;
+        print $sock $email;
+
+        seek( $spool, 0, 0 );
+        while ( my $line = <$spool> ) {
+            $line =~ s/\015?\012/\015\012/g;
+            $line =~ s/^\./\.\./g;
+            print $sock $line;
+        }
+
+    }
+    else {
+        my $body = $smtp->{'body'};
+        $body =~ s/\015?\012/\015\012/g;
+        $email .= $body;
+
+        # Handle transparency
+        $email =~ s/\015\012\./\015\012\.\./g;
+
+        print $sock $email;
+    }
 
     $self->logdebug( 'Sending end to destination' );
     $self->send_smtp_packet( $sock, '.',    '250' ) || return;
@@ -829,7 +1003,6 @@ sub close_destination_socket {
     $self->send_smtp_packet( $sock, 'QUIT', '221' ) || return;
     $sock->close();
     delete $smtp->{'destination_sock'};
-    return;
 }
 
 sub send_smtp_packet {
@@ -850,9 +1023,11 @@ sub send_smtp_packet {
     my $recv;
     eval {
         $recv = <$socket>;
+        $self->extra_debugging( "RAW DEBUG: ". $recv );
         while ( $recv =~ /^\d\d\d\-/ ) {
             $self->smtp_status('smtp.o.' . $status . '.waitext');
             $recv = <$socket>;
+            $self->extra_debugging( "RAW DEBUG: ". $recv );
         }
     };
     if ( my $error = $@ ) {
@@ -881,7 +1056,6 @@ sub add_header {
     my $smtp = $self->{'smtp'};
     $value =~ s/\015?\012/\015\012/g;
     push @{ $smtp->{'headers'} } , "$header: $value";
-    return;
 }
 
 sub change_header {
@@ -914,8 +1088,6 @@ sub change_header {
             #untested.
         }
     }
-
-    return;
 }
 
 sub insert_header {
@@ -923,20 +1095,23 @@ sub insert_header {
     my $smtp = $self->{'smtp'};
     $value =~ s/\015?\012/\015\012/g;
     splice @{ $smtp->{'headers'} }, $index - 1, 0, "$key: $value";
-    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Mail::Milter::Authentication::Protocol::SMTP - SMTP protocol specific methods
+Mail::Milter::Authentication::Protocol::SMTP - SMTP protocol handling
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-A Perl implenmetation of email authentication standards rolled up into a single easy to use milter.
+version 3.20240701
 
 =head1 SYNOPSIS
 
@@ -944,6 +1119,10 @@ Subclass of Net::Server::PreFork for bringing up the main server process for aut
 
 Please see Net::Server docs for more detail of the server code.
 
+=head1 DESCRIPTION
+
+A Perl implenmetation of email authentication standards rolled up into a single easy to use milter.
+
 =head1 FUNCTIONS
 
 =over
@@ -958,6 +1137,10 @@ Extract parameters from a SMTP command line.
 
 =over
 
+=item register_metrics
+
+Return details of the metrics this module exports.
+
 =item I<protocol_process_request( $command, $buffer )>
 
 Process the command from the SMTP protocol stream.
@@ -1051,21 +1234,15 @@ Update the process name status line
 
 =back
 
-=head1 DEPENDENCIES
+=head1 AUTHOR
 
-  English
-  Digest::MD5
-  Net::IP
+Marc Bradshaw <marc@marcbradshaw.net>
 
-=head1 AUTHORS
+=head1 COPYRIGHT AND LICENSE
 
-Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
-
-=head1 COPYRIGHT
-
-Copyright 2015
-
-This library is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+This software is copyright (c) 2020 by Marc Bradshaw.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
